local _t, _t2, _, T = {}, {}, ...

local function GetPetTypeIcon(petType)
	return "|TInterface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType] .. ":0:0:0:0:128:256:102:63:129:168|t"
end
local function SetZoomedIcon(self, tex)
	self:SetTexture(tex)
	self:SetTexCoord(6/64,60/64, 6/64,60/64)
end
local GetPetAbilities do
	local _t, _t2 = {}, {}
	function GetPetAbilities(species)
		wipe(_t)
		return C_PetJournal.GetPetAbilityList(species, _t, _t2)
	end
end
local GetFamilyColor do
	local c = {}
	for r,g,b in ("05a9fd 4baa14 dad15c a46d75 7c5a43 b479ff FCA800 f32f24 06bdc9 9b978b"):gmatch("(%x%x)(%x%x)(%x%x)") do
		c[#c+1], c[#c+2], c[#c+3] = tonumber(r,16)/255, tonumber(g,16)/255, tonumber(b,16)/255
	end
	function GetFamilyColor(id)
		return c[3*id-2], c[3*id-1], c[3*id]
	end
end
local GetAlternatePetAbilities do
	local idTable, levelTable = {}, {}
	function GetAlternatePetAbilities(sid, level, a1, a2, a3)
		wipe(idTable) wipe(levelTable)
		C_PetJournal.GetPetAbilityList(sid, idTable, levelTable)
		return level >= 10 and idTable[idTable[1] == a1 and 4 or 1],
			level >= 15 and idTable[idTable[2] == a2 and 5 or 2],
			level >= 20 and idTable[idTable[3] == a3 and 6 or 3]
	end
end

do -- LittleBattlerTip
	local function pickAnchor(self)
		local l, b, w, h = self:GetRect()
		local pl, pb, pw, ph = self:GetParent():GetRect()
	
		if (l - pl) < 0.2*pw then
			return "LEFT", self, "RIGHT", 5, 0
		elseif (l - pl) > 0.8*pw then
			return "RIGHT", self, "LEFT", -5, 0
		elseif (b - pb) < 0.5*ph then
			return "BOTTOM", self, "TOP", 0, -5
		else
			return "TOP", self, "BOTTOM", 0, 5
		end
	end
	function LittleBattlerTip:SetTamer(poi, name, title, tamer, eligible, completed)
		local p1, p2, p3, lvl, ql = T.GetTamerTeam(tamer)
		self:ClearAllPoints()
		self:SetPoint(pickAnchor(poi))
		self:SetOwner(poi, "ANCHOR_PRESERVE")
		self:AddLine(name, 1,1,1)
		if not p1 and (title or "") ~= "" and lvl and ql then
			title = "|c" .. select(4, GetItemQualityColor(ql-1)) .. "[" .. lvl .. "]|r " .. title
		end
		if (title or "") ~= "" then self:AddLine(title, nil, nil, nil, 1) end
		if p1 then self:AddLine(" ") end
		for i=1,p3 and 3 or p2 and 2 or p1 and 1 or 0 do
			local sid = i == 3 and p3 or i == 2 and p2 or p1
			local r, name, icon, ptype = "", C_PetJournal.GetPetInfoBySpeciesID(sid)
			local abilities = GetPetAbilities(sid)
			for i=1,#abilities do
				local i, _, _, _, _, _, pt, nsw = C_PetBattles.GetAbilityInfoByID(abilities[i])
				if not nsw then
					r = r .. " " .. GetPetTypeIcon(pt)
				end
			end
			self:AddDoubleLine(("%s |T%s:0:0:0:0:64:64:6:58:6:58|t |c%s[%d]|r %s"):format(
				GetPetTypeIcon(ptype), icon, select(4,GetItemQualityColor(ql-1)), lvl, name ~= "" and name or T.GetTamerPetName(sid)), r)
		end
		if completed then
			self:AddLine(" ")
			self:AddLine(GREEN_FONT_COLOR_CODE .. "|TInterface\\Scenarios\\ScenarioIcon-Check:0|t Daily quest completed")
		elseif not eligible then
			self:AddLine(" ")
			self:AddLine(GRAY_FONT_COLOR_CODE .. "|TInterface\\PetBattles\\PetBattle-LockIcon:0|t Daily not unlocked")
		elseif eligible == "faction" then
			self:AddLine(" ")
			self:AddLine(GRAY_FONT_COLOR_CODE .. "|TInterface\\PetBattles\\PetBattle-LockIcon:0|t Faction locked")
		end
		self:SetScale(UIParent:GetScale())
		self:Show()
	end
end

local function poiEnter(self)
	local tamer, eligible, completed, name, title
	if self.tamerId then
		tamer, eligible, completed = self.tamerId, T.GetTamerStatus(self.tamerId)
		name, title = T.GetTamerName(tamer)
	else
		name, title, tamer, eligible, completed = self.name, self.description, T.IsTamerLandmark(tonumber(self:GetName():match("%d+$")))
	end
	if tamer then
		WorldMapFrameAreaLabel:SetText("")
		WorldMapFrameAreaDescription:SetText("")
		LittleBattlerTip:SetTamer(self, name, title, tamer, eligible, completed)
	end
end
local function poiLeave(self)
	if LittleBattlerTip:IsOwned(self) then
		LittleBattlerTip:Hide()
	end
end
local function poiReset(self)
	self.Texture:SetAlpha(1)
	self.Texture:SetDesaturated(nil)
	self:SetFrameLevel(WorldMapButton:GetFrameLevel()+1)
end

local function createPOI(t, k)
	local button = CreateFrame("Button", nil, WorldMapButton)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnEnter", poiEnter)
	button:SetScript("OnLeave", poiLeave)

	button.Texture = button:CreateTexture()

	WorldMap_ResetPOI(button, isObjectIcon)
	t[k] = button
	return button
end
local ownPOIs = setmetatable({}, {__index=createPOI})

local function worldHide()
	local o = LittleBattlerTip:GetOwner()
	if o and o:GetParent() == WorldMapButton then
		LittleBattlerTip:Hide()
	end
end
local function updateLandmark(poi, tamer, eligible, completed)
	if eligible ~= true then
		poi.Texture:SetAlpha(eligible == "faction" and 0.4 or 0.6)
	end
	if completed then
		poi.Texture:SetDesaturated(1)
	end
	poi:SetFrameLevel(WorldMapButton:GetFrameLevel()+2)
end
local function addExtraMarks(i, tamer, x, y, ...)
	if tamer then
		local poi, _, _, w, h = ownPOIs[i], WorldMapButton:GetRect()
		poi:Show()
		poi:SetPoint("CENTER", WorldMapButton, "TOPLEFT", x/1e5*w, -y/1e5*h)
		poi.Texture:SetTexCoord(GetPOITextureCoords(196))
		poi.tamerId = tamer
		updateLandmark(poi, tamer, T.GetTamerStatus(tamer))
		return addExtraMarks(i+1, ...)
	else
		for i=i,#ownPOIs do
			ownPOIs[i]:Hide()
		end
	end
end
local function worldUpdate()
	for i=1,GetNumMapLandmarks() do
		local tamer, eligible, completed = T.IsTamerLandmark(i)
		if tamer then
			updateLandmark(_G["WorldMapFramePOI" .. i], tamer, eligible, completed)
		end
	end
	addExtraMarks(1, T.GetExtraLandmarks(GetCVarBool("showTamers") and GetCurrentMapAreaID()))
end

local LB_PrepareFrame
local function gossipShowInfo(fightId, lbTamerId)
	LB_PrepareFrame(LittleBattlerTeam2, T.GetTamerTeam(lbTamerId))
	LittleBattlerTeam2.fight.gossipId = fightId
	LittleBattlerTeam2.fight:SetText(fightId and "Let's battle!" or "Goodbye")
end
local function gossipUpdate()
	local id = T.IsTamerNPC(tonumber((UnitGUID("npc") or "0"):sub(6, 10), 16))
	if id and T.GetTamerTeam(id) then
		local last = _G["GossipTitleButton" .. (GossipFrame.buttonIndex-1)]
		if last and last.type == "Gossip" then
			gossipShowInfo(last:GetID(), id)
		else
			local button = _G["GossipTitleButton" .. GossipFrame.buttonIndex]
			local tex = _G[button:GetName() .. "GossipIcon"]
			tex:SetTexture([[Interface\GossipFrame\GossipGossipIcon]])
			button:SetID(GossipFrame.buttonIndex)
			button.type, button.lbTamerId = "LittleBattler", id
			button:SetText("Tell me more about your pets...")
			button:Show()
			GossipResize(button)
			GossipFrame.buttonIndex = GossipFrame.buttonIndex + 1
		end
	end
end
function LB_UpdateAbilityButton(self, id1, id2)
	local _, icon, atype = C_PetJournal.GetPetAbilityInfo(id1)
	self.id, self.id2 = id1, id2
	SetZoomedIcon(self.icon, icon)
	self.iconBorder:SetVertexColor(GetFamilyColor(atype))
	self:GetHighlightTexture():SetVertexColor(GetFamilyColor(atype))
	if self:IsMouseOver() and self:GetScript("OnEnter") then self:GetScript("OnEnter")(self) end
end
local function LB_UpdateOwnPets(self)
	local icof = "|TInterface\\PetBattles\\PetBattle-StatIcons:0:0:0:0:32:32:%d:%d:%d:%d|t %s"
	for i=1,3 do
		local slot = self.team[i]
		local pid, a1, a2, a3, locked = C_PetJournal.GetPetLoadOutInfo(i)
		slot:SetShown(not not pid)
		if pid then
			local sid, cn, lvl, _, _, _, _, n, ico, ptype = C_PetJournal.GetPetInfoByPetID(pid)
			local hp, mhp, pow, spd, ql = C_PetJournal.GetPetStats(pid)
			local b1, b2, b3 = GetAlternatePetAbilities(sid, lvl, a1, a2, a3)
			slot.iconButton.id = PET_BATTLE_PET_TYPE_PASSIVES[ptype]
			slot.iconButton.dead:SetShown(hp == 0)
			slot.iconBorder:SetVertexColor(GetFamilyColor(ptype))
			slot.levelBG:SetVertexColor(GetFamilyColor(ptype))
			slot.level:SetText("|c" .. select(4, GetItemQualityColor(ql - 1)) .. (lvl or "?"))
			slot.name:SetText(cn or n or "?")
			slot.stats:SetFormattedText("%s  %s", icof:format(16,32,16,32, mhp > hp and (hp .. " / " .. mhp) or hp), icof:format(0,16,16,32, spd))
			slot.id, slot.lvl, slot.ap, slot.sp, slot.hp, slot.mhp = sid, lvl, pow, spd, hp, mhp
			SetZoomedIcon(slot.icon, ico)
			for j=1,#slot.abilities do
				local aid = j == 1 and a1 or j == 2 and a2 or j == 3 and a3
				local bid = j == 1 and b1 or j == 2 and b2 or j == 3 and b3
				LB_UpdateAbilityButton(slot.abilities[j], aid, bid)
			end
		end
	end
end
function LB_PrepareFrame(self, p1, p2, p3, lvl, ql)
	local numPets = p3 and 3 or p2 and 2 or 1
	
	local icof = "|TInterface\\PetBattles\\PetBattle-StatIcons:0:0:0:0:32:32:%d:%d:%d:%d|t %s\n"
	for i=1,numPets do
		local pid, pet = i == 3 and p3 or i == 2 and p2 or p1, self.pets[i]
		local name, ico, ptype = C_PetJournal.GetPetInfoBySpeciesID(pid)
		local hp, ap, sp, rlvl = T.GetTamerPetStats(pid)
		pet:Show()
		pet.name:SetText((name or "") ~= "" and name or T.GetTamerPetName(pid))
		pet.level:SetText("|c" .. select(4, GetItemQualityColor((ql or 2)-1)) .. (rlvl or lvl or "?"))
		pet.iconButton.id = PET_BATTLE_PET_TYPE_PASSIVES[ptype]
		SetZoomedIcon(pet.icon, ico)
		pet.iconBorder:SetVertexColor(GetFamilyColor(ptype))
		pet.levelBG:SetVertexColor(GetFamilyColor(ptype))
		pet.stats:SetFormattedText("%s%s%s", hp and icof:format(16,32,16,32, hp) or "", sp and icof:format(0,16,16,32, sp) or "", ap and icof:format(0,16,0,16, ap) or "")
		pet.id, pet.lvl, pet.ap, pet.sp, pet.hp, pet.mhp = pid, rlvl or lvl, ap, sp, hp, hp
		local aid = GetPetAbilities(pid)
		for j=1,#aid do
			local ab, name, icon, atype = pet.abilities[j], C_PetJournal.GetPetAbilityInfo(aid[j])
			SetZoomedIcon(ab.icon, icon)
			ab.iconBorder:SetVertexColor(GetFamilyColor(atype))
			ab.id = aid[j]
		end
	end
	for i=numPets+1,3 do
		self.pets[i]:Hide()
	end
	LB_UpdateOwnPets(self)
	
	self:Show()
end
T.Evie.RegisterEvent("UPDATE_SUMMONPETS_ACTION",function()
	if LittleBattlerTeam2:IsShown() then
		LB_UpdateOwnPets(LittleBattlerTeam2)
	end
end)
local function gossipSelect(id)
	local btn = _G["GossipTitleButton" .. id]
	if btn and btn.type == "LittleBattler" then
		gossipShowInfo(nil, btn.lbTamerId)
	end
end
local function gossipConfirm(_, event, gid)
	if event == "GOSSIP_CONFIRM" and T.IsTamerNPC(tonumber((UnitGUID("npc") or "0"):sub(6, 10), 16)) then
		T.Evie.RaiseEvent("PET_BATTLE_SOON")
		SelectGossipOption(gid, "", true)
		StaticPopup_Hide(event)
	end
end
local function gossipCleanup()
	local i, b = 2, GossipTitleButton1
	while b do
		b.lbTamerId = nil
		i, b = i + 1, _G["GossipTitleButton" .. i]
	end
end

local function OnUnit(self)
	local g = UnitGUID(select(2,self:GetUnit()) or "none") or "0x0"
	local id = tonumber(g:sub(6, 10), 16)
	local sid, qid, lbid = T.IsFableNPC(id)
	if sid then
		local hp, pwr, spd = T.GetTamerPetStats(sid)
		local icof = "|TInterface\\PetBattles\\PetBattle-StatIcons:0:0:0:0:32:32:%d:%d:%d:%d|t %s"
		if hp and spd and pwr then
			self:AddLine(icof:format(16,32,16,32, hp) .. "   " .. icof:format(0,16,16,32, spd) .. "   " .. icof:format(0,16,0,16, pwr), HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
			self:AddLine(" ")
		end
		local abilities = GetPetAbilities(sid)
		for i=1,#abilities do
			local i, name, _, _, _, _, pt, nsw = C_PetBattles.GetAbilityInfoByID(abilities[i])
			self:AddLine(GetPetTypeIcon(pt) .. " " .. name)
		end
		self:Show()
	end
end

WorldMapPlayerUpper:EnableMouse(nil)
WorldMapFrame:HookScript("OnHide", worldHide)
GameTooltip:HookScript("OnTooltipSetUnit", OnUnit)
hooksecurefunc("WorldMapFrame_Update", worldUpdate)
hooksecurefunc("WorldMapFrame_UpdateMap", worldUpdate)
hooksecurefunc("WorldMapPOI_OnEnter", poiEnter)
hooksecurefunc("WorldMapPOI_OnLeave", poiLeave)
hooksecurefunc("WorldMap_ResetPOI", poiReset)
hooksecurefunc("GossipFrameUpdate", gossipUpdate)
hooksecurefunc("SelectGossipOption", gossipSelect)
UIParent:HookScript("OnEvent", gossipConfirm)
GossipFrameGreetingPanel:HookScript("OnHide", gossipCleanup)
local i, f = 2, WorldMapFramePOI1 while f do
	f:HookScript("OnEnter", poiEnter)
	f:HookScript("OnLeave", poiLeave)
	i, f = i + 1, _G["WorldMapFramePOI" .. i]
end